function [A,b,u] = FiniteElement(X,coeff,f,bc,doPlot)

% print usage
if nargin ~= 5
  ME = MException('FiniteElement:MissingArguments',['Usage: FiniteElement(X,coeff,f,bc,plot)\n'...
  'X is the vector of nodes\n'...
  'coeff is the vector [a c] of the coefficients of the ODE\n'...
  'f is the source term of the ODE (function handle)\n'...
  'bc is the vector describing the BC of the BVP [BC type at x_0 (bool), q0 or p0, BC type at x_n (bool), q1 or p1]\n'...
  'doPlot (bool)\n']);
  throw(ME);
end

% check arguments
if ~isvector(X) || ~isnumeric(X) || ~isreal(X) || length(X) < 2
  ME = MException('FiniteElement:InconsistentArguments','X must be a real vector');
  throw(ME);
end

if ~all(diff(X)>0)
  ME = MException('FiniteElement:InconsistentArguments','X must be a monotonically increasing vector');
  throw(ME);
end

if ~isvector(coeff) || ~isnumeric(coeff) || ~isreal(coeff) || length(coeff) ~= 2
  ME = MException('FiniteElement:InconsistentArguments','coeff must be a real vector of length 2');
  throw(ME);
end

if ~isa(f,'function_handle')
  ME = MException('FiniteElement:InconsistentArguments','f must be a function');
  throw(ME);
end

if ~isvector(bc) || ~isnumeric(bc) || ~isreal(bc) || length(bc) ~= 4
  ME = MException('FiniteElement:InconsistentArguments','bc must be a real vector of length 4');
  throw(ME);
end

if ~isscalar(doPlot) || ~isnumeric(doPlot) || ~isreal(doPlot)
  ME = MException('FiniteElement:InconsistentArguments','plot must be a scalar (bool)');
  throw(ME);
end

n = length(X)-1;
a = coeff(1);
c = coeff(2);
M = MassMatrix(X);
S = StiffnessMatrix(X);
u = zeros(n+1,1);
fvec = zeros(n+1,1);
for i = 1:length(X)
  fvec(i) = f(X(i));
end

b0 = M*fvec;

if bc(1)
  i0 = 1;
  q0 = bc(2);
  b0(1) = b0(1)-a*q0;
else
  i0 = 2;
  p0 = bc(2);
  u(1) = p0;
  b0(2) = b0(2)-(a*S(2,1)+c*M(2,1))*p0;
end

if bc(3)
  i1 = n+1;
  q1 = bc(4);
  b0(n+1) = b0(n+1)+a*q1;
else
  i1 = n;
  p1 = bc(4);
  u(n+1) = p1;
  b0(n) = b0(n)-(a*S(n,n+1)+c*M(n,n+1))*p1;
end

% construct the matrix of the equation
A = a*S(i0:i1,i0:i1)+c*M(i0:i1,i0:i1);

% construct the rhs of the equation
b = b0(i0:i1);

% solve the equation
u(i0:i1) = A\b;

% plot if needed
if doPlot
  plot(X,u);
end
