function u = ParabolicFEM(X,coeff,f,bc,u0,tau,nStep,method,verbose)
% u_t = -au_{xx} + cu + f(x)
% print usage
if nargin ~= 9
  ME = MException('ParabolicFEM:MissingArguments',['Usage: ParabolicFEM(X,coeff,f,bc,u0,tau,nStep,method,verbose)\n'...
  'X is the vector of nodes\n'...
  'coeff is the vector [a c] of the coefficients of the ODE\n'...
  'f is the source term of the ODE (function handle)\n'...
  'bc is the 2-component vector of BCs [p0, p1]\n'...
  'u0 is a vector of the initial state\n'...
  'tau is a the time-step\n'...
  'nStep is the number of steps to take\n'...
  'method is a string; valid values are FE or BE\n'...
  'verbose is the level of verbosity\n']);
  throw(ME);
end

% check arguments
if ~isvector(X) || ~isnumeric(X) || ~isreal(X) || length(X) < 2
  ME = MException('FiniteElement:InconsistentArguments','X must be a real vector');
  throw(ME);
end

if ~all(diff(X)>0)
  ME = MException('FiniteElement:InconsistentArguments','X must be a vector of increasing elements');
  throw(ME);
end

if ~isvector(coeff) || ~isnumeric(coeff) || ~isreal(coeff) || length(coeff) ~= 2
  ME = MException('FiniteElement:InconsistentArguments','coeff must be a real vector of length 2');
  throw(ME);
end

if ~isa(f,'function_handle')
  ME = MException('FiniteElement:InconsistentArguments','f must be a function');
  throw(ME);
end

if ~isvector(bc) || ~isnumeric(bc) || ~isreal(bc) || length(bc) ~= 2
  ME = MException('FiniteElement:InconsistentArguments','bc must be a real vector of length 2');
  throw(ME);
end

if ~isvector(u0) || ~isnumeric(u0) || ~isreal(u0) || length(u0) ~= length(X)
  ME = MException('FiniteElement:InconsistentArguments','u0 must be a real vector of the same size as X');
  throw(ME);
end

if ~isscalar(tau) || ~isnumeric(tau) || ~isreal(tau) || tau <= 0
  ME = MException('FiniteElement:InconsistentArguments','tau must be a positive scalar');
  throw(ME);
end

if ~isscalar(nStep) || ~isnumeric(nStep) || ~isreal(nStep) || nStep < 0
  ME = MException('FiniteElement:InconsistentArguments','nStep must be a positive scalar');
  throw(ME);
end

if ~ischar(method) || method ~= 'FE' || method ~= 'BE'
  ME = MException('FiniteElement:InconsistentArguments','method must be either FE or BE');
  throw(ME);
end

if ~isscalar(verbose) || ~isnumeric(verbose) || ~isreal(verbose)
  ME = MException('FiniteElement:InconsistentArguments','verbose must be a scalar');
  throw(ME);
end

maxPlot = 10;
n = length(X)-1;
a = coeff(1);
c = coeff(2);
p0 = bc(1);
p1 = bc(2);
M = MassMatrix(X);
S = StiffnessMatrix(X);
nPlot = ceil(nStep/maxPlot);

uold = reshape(u0,[n+1,1]);

fvec = zeros(n+1,1);
for i = 1:length(X)
  fvec(i) = f(X(i));
end

% construct the matrix of the equation
A0 = zeros(1,n+1);
A0(1) = 1;
An = zeros(1,n+1);
An(n+1) = 1;

if method == 'FE'
  A = M/tau;
elseif method == 'BE'
  A = M/tau - a*S - c*M;
end

A = A(2:n,:);
A = [A0; A; An];

% construct the matrices needed for the lhs of the equation
F = M*fvec;
F(1) = p0;
F(n+1) = p1;

if method == 'FE'
  B = (M/tau + a*S + c*M);
elseif method == 'BE'
  B = M/tau;
end

B(1,:) = zeros(1,n+1);
B(n+1,:) = zeros(1,n+1);

% get the stationary solution to determine the range
ustat = (A-B)\F;
Hmin = 1.1*min(min(ustat),min(u0));
Hmax = 1.1*max(max(ustat),max(u0));

% solve the differential equation
for i = [1:1:nStep]
  u = A\(B*uold + F);
  uold = u;
  if verbose >= 1 && mod(i,nPlot) == 0
    plot(reshape(X,[1,n+1]), reshape(u,[1,n+1]), '-o');
    axis([X(1), X(n+1), Hmin, Hmax]);
    drawnow;
    if verbose >= 2
      reshape(u,[1,n+1])
    end
  end
end

% output
u = reshape(uold,size(u0));
if verbose >= 2
  A
  B
  F
  u
end
